/*
 * Apaq Framework
 * Copyright (C) 2011-2013 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria;

/**
 * Defines a Criteria for resolving data. It contains a <code>Rule</code>, <code>Limit</code> and a <code>Sorter</code> of which all are optional.
 */
public class Criteria {
    
    private final Rule rule;
    private final Limit limit;
    private final Sorter sorter;

    public Criteria() {
        this(null, null, null);
    }
    
    

    public Criteria(Rule rule) {
        this(rule, null, null);
    }
    
    public Criteria(Limit limit) {
        this(null, null, limit);
    }
    
    public Criteria(Sorter sorter) {
        this(null, sorter, null);
    }
    
    public Criteria(Rule rule, Limit limit) {
        this(rule, null, limit);
    }

    public Criteria(Rule rule, Sorter sorter) {
        this(rule, sorter, null);
    }
    
    public Criteria(Sorter sorter, Limit limit) {
        this(null, sorter, limit);
    }

    public Criteria(Rule rule, Sorter sorter, Limit limit) {
        this.rule = rule;
        this.limit = limit;
        this.sorter = sorter;
    }

    public Limit getLimit() {
        return limit;
    }

    public Rule getRule() {
        return rule;
    }

    public Sorter getSorter() {
        return sorter;
    }
    
    
}
